@@ -4,6 +4,7 @@ import logging
 
 from alephclient import settings
 from alephclient.api import AlephAPI
+from alephclient.util import prop_push
 from alephclient.errors import AlephException
 from alephclient.tasks.crawldir import crawl_dir
 from alephclient.tasks.bulkload import bulk_load
@@ -98,8 +99,9 @@ def write_entities(ctx, infile, foreign_id, force=False, unsafe=False):
 @click.option('-o', '--outfile', type=click.File('w'), default='-')  # noqa
 @click.option('-s', '--schema', multiple=True, default=[])  # noqa
 @click.option('-f', '--foreign-id', help="foreign_id of the collection")
+@click.option('-p', '--publisher', is_flag=True, default=False, help="Add publisher info from collection context")  # noqa
 @click.pass_context
-def stream_entities(ctx, outfile, schema, foreign_id):
+def stream_entities(ctx, outfile, schema, foreign_id, publisher):
     """Load entities from the server and print them to stdout."""
     api = ctx.obj["api"]
     try:
@@ -107,9 +109,31 @@ def stream_entities(ctx, outfile, schema, foreign_id):
         collection = api.get_collection_by_foreign_id(foreign_id)
         if collection is None:
             raise click.BadParameter("Collection %r not found!" % foreign_id)
+        publisher_label = collection.get('publisher', collection.get('label'))
+        publisher_url = collection.get('links', {}).get('ui')
+        publisher_url = collection.get('publisher_url', publisher_url)
         for entity in api.stream_entities(collection_id=collection.get('id'),
-                                          include=include, schema=schema,
-                                          decode_json=False):
+                                          include=include, schema=schema):
+            if publisher:
+                prop_push(entity, 'publisher', publisher_label)
+                prop_push(entity, 'publisherUrl', publisher_url)
+            outfile.write(json.dumps(entity))
+            outfile.write('\n')
+    except AlephException as exc:
+        raise click.ClickException(exc.message)
+    except BrokenPipeError:
+        raise click.Abort()
+
+
+@cli.command('linkages')
+@click.option('-o', '--outfile', type=click.File('w'), default='-')  # noqa
+@click.option('-c', '--context', multiple=True, default=[])  # noqa
+@click.pass_context
+def linkages(ctx, outfile, context):
+    """Stream all linkages within the given role contexts."""
+    api = ctx.obj["api"]
+    try:
+        for entity in api.linkages(context_ids=context):
             outfile.write(json.dumps(entity))
             outfile.write('\n')
     except AlephException as exc:
